
#include <rapp/rapp.h>
#include <stdio.h>
#include <string.h>

void
run_gather_scatter(uint8_t *src_buf, int src_dim, uint8_t *map_buf, int map_dim, int width, int height, int rows)
{

    int      pack_dim = rapp_align(width*height);
    uint8_t *pack_buf = rapp_malloc(pack_dim * rows, 0);
    memset(pack_buf, 0, pack_dim * rows);

    rapp_gather_u8(pack_buf, pack_dim, src_buf, src_dim, map_buf, map_dim, width, height, rows);
    rapp_pixop_addc_u8(pack_buf, pack_dim, pack_dim * rows, rows, 1);
    rapp_scatter_u8(src_buf, src_dim, map_buf, map_dim,
                    pack_buf, width, height);
    rapp_free(pack_buf);
}

void
run_cond_addc(uint8_t *src_buf, int src_dim, uint8_t *map_buf, int map_dim, int width, int height)
{
    rapp_cond_addc_u8(src_buf, src_dim, map_buf, map_dim, width, height, 1);
}

static void
main_init()
{
    rapp_initialize();
}

static void
main_close()
{
    rapp_terminate();
}

int main(int argc, char **argv)
{
    main_init();

    int width = 320;
    int height = 180;
    int rows = 1;
    int src_dim = rapp_align(width);
    int map_dim = rapp_align((width + 7) / 8);

    uint8_t *src_buf = rapp_malloc(src_dim * height, 0);
    memset(src_buf, 0x2, src_dim * height);

    uint8_t *src2_buf = rapp_malloc(src_dim * height, 0);
    memset(src2_buf, 0x2, src_dim * height);

    uint8_t *map_buf = rapp_malloc(map_dim * height, 0);

    /* Full */
    memset(map_buf, 0xFF, map_dim * height);

    run_gather_scatter(src_buf, src_dim, map_buf, map_dim, width, height, rows);

    run_cond_addc(src2_buf, src_dim, map_buf, map_dim, width, height);

    if (memcmp(src_buf, src2_buf, src_dim * height) != 0) {
        printf("Failed\n");
    }

    /* checker (almost "full") */
    memset(map_buf, 0xAA, map_dim * height);

    run_gather_scatter(src_buf, src_dim, map_buf, map_dim, width, height, rows);

    run_cond_addc(src2_buf, src_dim, map_buf, map_dim, width, height);

    if (memcmp(src_buf, src2_buf, src_dim * height) != 0) {
        printf("Failed\n");
    }

    /* Empty */
    memset(map_buf, 0x0, map_dim * height);

    run_gather_scatter(src_buf, src_dim, map_buf, map_dim, width, height, rows);

    run_cond_addc(src2_buf, src_dim, map_buf, map_dim, width, height);

    if (memcmp(src_buf, src2_buf, src_dim * height) != 0) {
        printf("Failed\n");
    }

    rapp_free(src_buf);
    rapp_free(src2_buf);
    rapp_free(map_buf);

    main_close();

    return 0;
}

